@@ -21,6 +21,7 @@ void free_bios_devices(void *cookie)
 		return;
 	list_for_each_entry_safe(dev, n, &state->bios_devices, node) {
 		list_del(&(dev->node));
+		free(dev->bios_name);
 		free(dev);
 	}
 }
@@ -138,6 +139,7 @@ static int sort_smbios(const struct bios_device *x, const struct bios_device *y)
 
 	if      (x->pcidev && !y->pcidev) return -1;
 	else if (!x->pcidev && y->pcidev) return 1;
+	else if (!x->pcidev && !y->pcidev) return 0;
 
 	a = x->pcidev;
 	b = y->pcidev;
@@ -212,33 +214,17 @@ static void sort_device_list(struct libbiosdevname_state *state)
 	list_splice(&sorted_devices, &state->bios_devices);
 }
 
-static void match_pci_and_eth_devs(struct libbiosdevname_state *state)
+/* Check for Mellanox/Chelsio drivers */
+int ismultiport(const char *driver)
 {
-	struct pci_device *p;
-	struct bios_device *b;
-	struct network_device *n;
-
-	list_for_each_entry(n, &state->network_devices, node) {
-		p = find_dev_by_pci_name(state, n->drvinfo.bus_info);
-		if (!p)
-			continue;
-
-		b = malloc(sizeof(*b));
-		if (!b)
-			continue;
-		memset(b, 0, sizeof(*b));
-		INIT_LIST_HEAD(&b->node);
-		b->pcidev = p;
-		b->netdev = n;
-		b->slot_num = -1;
-		b->port_num = -1;
-		claim_netdev(b->netdev);
-		list_add(&b->node, &state->bios_devices);
-	}
+	if (!strncmp(driver, "mlx4", 4))
+		return 1;
+	if (!strncmp(driver, "cxgb", 4))
+		return 1;
+	return 0;
 }
 
-
-static void match_eth_and_pci_devs(struct libbiosdevname_state *state)
+static void match_pci_and_eth_devs(struct libbiosdevname_state *state)
 {
 	struct pci_device *p;
 	struct bios_device *b;
@@ -249,22 +235,36 @@ static void match_eth_and_pci_devs(struct libbiosdevname_state *state)
 		if (!is_pci_network(p))
 			continue;
 
+		/* Loop through all ether devices to find match */
 		unparse_pci_name(pci_name, sizeof(pci_name), p->pci_dev);
-		n = find_net_device_by_bus_info(state, pci_name);
-		if (!n)
-			continue;
-
-		b = malloc(sizeof(*b));
-		if (!b)
-			continue;
-		memset(b, 0, sizeof(*b));
-		INIT_LIST_HEAD(&b->node);
-		b->pcidev = p;
-		b->netdev = n;
-		b->slot_num = -1;
-		b->port_num = -1;
-		claim_netdev(b->netdev);
-		list_add(&b->node, &state->bios_devices);
+		list_for_each_entry(n, &state->network_devices, node) {
+			//Accept only Ethernet devices, otherwise ignore.
+			if(!netdev_arphrd_type_is_eth(n))
+				continue;
+			if (strncmp(n->drvinfo.bus_info, pci_name, sizeof(n->drvinfo.bus_info)))
+				continue;
+			/* Ignore if devtype is fcoe */
+			if (netdev_devtype_is_fcoe(n))
+				continue;
+			b = malloc(sizeof(*b));
+			if (!b)
+				continue;
+			memset(b, 0, sizeof(*b));
+			INIT_LIST_HEAD(&b->node);
+			b->pcidev = p;
+			b->netdev = n;
+			b->port = NULL;
+			if (ismultiport(n->drvinfo.driver)) {
+				b->port = malloc(sizeof(struct pci_port));
+				if (b->port != NULL) {
+					b->port->port = n->devid+1;
+					b->port->pfi = p->is_sriov_virtual_function ?
+						p->vf_index : -1;
+				}
+			}
+			claim_netdev(b->netdev);
+			list_add(&b->node, &state->bios_devices);
+		}
 	}
 }
 
@@ -280,14 +280,16 @@ static void match_unknown_eths(struct libbiosdevname_state *state)
 			continue;
 		if (!is_ethernet(n)) /* for virtual interfaces */
 			continue;
+		/* Ignore if devtype is fcoe */
+		if (netdev_devtype_is_fcoe(n))
+			continue;
 		b = malloc(sizeof(*b));
 		if (!b)
 			continue;
 		memset(b, 0, sizeof(*b));
 		INIT_LIST_HEAD(&b->node);
 		b->netdev = n;
-		b->slot_num = -1;
-		b->port_num = -1;
+		b->port = NULL;
 		list_add(&b->node, &state->bios_devices);
 	}
 }
@@ -355,26 +357,6 @@ static void find_duplicates(struct libbiosdevname_state *state)
 	}
 }
 
-extern int addslot(struct libbiosdevname_state *state, int slot);
-
-/* Fix for RHBZ 816536/757743/756164/: Cards with same PCI but multiple ports
- * chelsio, mellanox */
-static void check_ports(struct libbiosdevname_state *state)
-{
-	struct pci_device *dev;
-	struct bios_device *a;
-
-	list_for_each_entry(a, &state->bios_devices, node) {
-		dev = a->pcidev;
-		if (dev == NULL || dev->is_sriov_virtual_function || dev->vpd_port != INT_MAX)
-			continue;
-		if (dev->physical_slot != PHYSICAL_SLOT_UNKNOWN) {
-			a->slot_num = dev->physical_slot;
-			a->port_num = addslot(state, 0x1000 + dev->physical_slot);
-		}
-	}
-}
-
 void * setup_bios_devices(int namingpolicy, const char *prefix)
 {
 	int rc=1;
@@ -390,7 +372,6 @@ void * setup_bios_devices(int namingpolicy, const char *prefix)
 	get_eths(state);
 	match_all(state);
 	sort_device_list(state);
-	check_ports(state);
 	rc = assign_bios_network_names(state, namingpolicy, prefix);
 	if (rc)
 		goto out;
