@@ -20,6 +20,12 @@ struct slotlist
 	int count;
 };
 
+struct pci_port {
+	struct list_head node;
+	int port;
+	int pfi;
+};
+
 struct pci_device {
 	struct list_head node;
 	struct pci_dev *pci_dev;
@@ -27,6 +33,7 @@ struct pci_device {
 	unsigned int index_in_slot; /* only valid if physical_slot > 0 and not a VF */
 	unsigned int embedded_index; /* only valid if embedded_index_valid */
 	unsigned short int class;
+  	unsigned int  sbus;
 	unsigned char uses_smbios;
 	unsigned char smbios_type;
 	unsigned char smbios_instance;
@@ -43,6 +50,7 @@ struct pci_device {
 	struct pci_device *pf;
 	struct list_head vfnode;
 	struct list_head vfs;
+	struct list_head ports;
 	unsigned int is_sriov_physical_function:1;
 	unsigned int is_sriov_virtual_function:1;
 	unsigned int embedded_index_valid:1;
@@ -94,5 +102,7 @@ static inline int pci_domain_nr(const struct pci_dev *dev)
 }
 #endif
 
+int is_root_port(const struct libbiosdevname_state *state,
+		 int domain, int bus, int device, int func);
 
 #endif /* PCI_H_INCLUDED */
